/*******************************************************************
HILBEX
version 0.1, 1 Oct 2016

This file is part of the software package Hilbex by Herman
Haverkort. Use and distribution of this software is subject to
restrictions. Consult readme.txt for details, or download it from
http://spacefillingcurves.net/.
*******************************************************************/

Partition::Partition(const char c, const OPt& p0, const OPt& p1, const OPt& p2, const OPt& p3, const char* t, const char* p):
  symbol(c), transformations(t), permutations(p)
{
  pattern[0] = p0; pattern[1] = p1; pattern[2] = p2; pattern[3] = p3;
}

int Permutation::operator()(const int i) const
{
  return permut[i];
}

int Permutation::inv(const int i) const
{
  return invper[i];
}

Permutation::Permutation(const char c, const int p0, const int p1, const int p2, const int p3):
  symbol(c)
{
  invper[0] = p0-1; permut[invper[0]] = 0;
  invper[1] = p1-1; permut[invper[1]] = 1;
  invper[2] = p2-1; permut[invper[2]] = 2;
  invper[3] = p3-1; permut[invper[3]] = 3;
}      

bool GateType::facetGate() const { return (location[0] + location[1] + location[2] == 2); }
bool GateType::edgeGate() const { return (location[0] + location[1] + location[2] == 1); }
bool GateType::vertexGate() const { return (location == NPt(0,0,0)); }
int GateType::operator[](const int i) const { return location[i]; }
GateType::GateType(const char c, const NPt& p):
  symbol(c), location(p)
{};

void initializeBasePatternTable()
{
  partitions.push_back(Partition('C', OPt(-1,-1,-1), OPt( 1,-1,-1), OPt( 1, 1,-1), OPt(-1, 1,-1), "adeklou", "012"));
  partitions.push_back(Partition('L', OPt(-1,-1,-1), OPt( 1,-1,-1), OPt(-1, 1, 1), OPt( 1, 1,-1), "al", "012345678cde"));
  partitions.push_back(Partition('N', OPt(-1,-1,-1), OPt( 1,-1,-1), OPt( 1, 1, 1), OPt(-1, 1, 1), "ae", "012345"));
  partitions.push_back(Partition('S', OPt(-1,-1,-1), OPt( 1,-1,-1), OPt( 1, 1,-1), OPt( 1, 1, 1), "ei", "0123456789ab"));
  partitions.push_back(Partition('X', OPt(-1,-1,-1), OPt( 1, 1,-1), OPt(-1, 1, 1), OPt( 1,-1, 1), "abcghijkloqrswxyz", "0"));
  partitions.push_back(Partition('Y', OPt(-1,-1,-1), OPt( 1,-1,-1), OPt( 1, 1,-1), OPt( 1,-1, 1), "hikoz", "0236"));
  permutations.push_back(Permutation('0', 1, 2, 3, 4));
  permutations.push_back(Permutation('1', 1, 2, 4, 3));
  permutations.push_back(Permutation('2', 1, 3, 2, 4));
  permutations.push_back(Permutation('3', 1, 3, 4, 2));
  permutations.push_back(Permutation('4', 1, 4, 2, 3));
  permutations.push_back(Permutation('5', 1, 4, 3, 2));
  permutations.push_back(Permutation('6', 2, 1, 3, 4));
  permutations.push_back(Permutation('7', 2, 1, 4, 3));
  permutations.push_back(Permutation('8', 2, 3, 1, 4));
  permutations.push_back(Permutation('9', 2, 3, 4, 1));
  permutations.push_back(Permutation('a', 2, 4, 1, 3));
  permutations.push_back(Permutation('b', 2, 4, 3, 1));
  permutations.push_back(Permutation('c', 3, 1, 2, 4));
  permutations.push_back(Permutation('d', 3, 1, 4, 2));
  permutations.push_back(Permutation('e', 3, 2, 1, 4));
  gateTypes.push_back(GateType('c', NPt(0,0,0)));
  gateTypes.push_back(GateType('f', NPt(0,1,1)));
  gateTypes.push_back(GateType('g', NPt(1,0,1)));
  gateTypes.push_back(GateType('r', NPt(1,0,0)));
  gateTypes.push_back(GateType('s', NPt(1,1,0)));
  gateTypes.push_back(GateType('t', NPt(0,0,1)));
  gateTypes.push_back(GateType('v', NPt(0,1,0)));
}

