/*******************************************************************
HILBEX
version 0.1, 1 Oct 2016

This file is part of the software package Hilbex by Herman
Haverkort. Use and distribution of this software is subject to
restrictions. Consult readme.txt for details, or download it from
http://spacefillingcurves.net/.
*******************************************************************/

FPt::FPt(const double x0, const double x1, const double x2) { x[0] = x0; x[1] = x1; x[2] = x2; }
double& FPt::operator[](const int i) { return x[i]; }
double FPt::operator[](const int i) const { return x[i]; }

ostream& operator<<(ostream& o, const FPt& p) { return (o << "< " << p[0] << ", " << p[1] << ", " << p[2] << " >"); }

NPt::NPt(const int x0, const int x1, const int x2) { x[0] = x0; x[1] = x1; x[2] = x2; }
int& NPt::operator[](const int i) { return x[i]; }
int NPt::operator[](const int i) const { return x[i]; }
double spread(const int x, const double scaleFactor)
{
  if (x == 0) return 0;
  return (scaleFactor * spread(x >> 1, scaleFactor) + (double) (x & 1));
}
FPt NPt::povRay(const double gap, const int order) const 
{ 
  const double scaleFactor = 2.0 / (1.0 - gap);
  double widthCorrection = 0;
  for (int i = 0; i < order; ++i) widthCorrection = scaleFactor * widthCorrection + 1.0;
  widthCorrection += 1.0;
  for (int i = 0; i < order; ++i) widthCorrection *= 0.5;
  return FPt(
    (spread(x[2],scaleFactor) + 0.5) / widthCorrection,
    (spread(x[1],scaleFactor) + 0.5) / widthCorrection,
    (spread(x[0],scaleFactor) + 0.5) / widthCorrection
  );
}

ostream& operator<<(ostream& o, const NPt& p) { return (o << '(' << p[0] << ',' << p[1] << ',' << p[2] << ')'); }

OPt::OPt(const int x0, const int x1, const int x2) { x[0] = x0; x[1] = x1; x[2] = x2; }
int& OPt::operator[](const int i) { return x[i]; }
int OPt::operator[](const int i) const { return x[i]; }
OPt::operator GPt() const { return GPt(x[0]*3, x[1]*3, x[2]*3); }

OPt convert(const NPt& p, const int order) { const int maxCoord = (1 << order) - 1; return OPt(p[0]*2 - maxCoord, p[1]*2 - maxCoord, p[2]*2 - maxCoord); }
NPt convert(const OPt& p, const int order) { const int maxCoord = (1 << order) - 1; return NPt((p[0]+maxCoord)/2, (p[1]+maxCoord)/2, (p[2]+maxCoord)/2); }

GPt::GPt(const int x0, const int x1, const int x2) { x[0] = x0; x[1] = x1; x[2] = x2; }
int& GPt::operator[](const int i) { return x[i]; }
int GPt::operator[](const int i) const { return x[i]; }

ostream& operator<<(ostream& o, const GPt& p) { return (o << '(' << p[0] << "/6," << p[1] << "/6," << p[2] << "/6)"); }

template <class P> P operator+(const P& p, const P& q) { return P(p[0]+q[0], p[1]+q[1], p[2]+q[2]); }
template <class P> P operator-(const P& p, const P& q) { return P(p[0]-q[0], p[1]-q[1], p[2]-q[2]); }
template <class P> P operator*(const P& p, const int s) { return P(p[0]*s, p[1]*s, p[2]*s); }
template <class P> P operator*(const P& p, const double s) { return P(p[0]*s, p[1]*s, p[2]*s); }
template <class P> P operator/(const P& p, const int s) { return P(p[0]/s, p[1]/s, p[2]/s); };
template <class P> bool operator==(const P& p, const P& q) { return (p[0] == q[0] && p[1] == q[1] && p[2] == q[2]); }
template <class P> bool operator!=(const P& p, const P& q) { return (!(p == q)); }
template <class P> bool coplanar(const P& p, const P& q) { return (p[0] == q[0] || p[1] == q[1] || p[2] == q[2]); }
template <class P> bool collinear(const P& p, const P& q) { return ((p[0]==q[0]?1:0) + (p[1]==q[1]?1:0) + (p[2]==q[2]?1:0) >= 2); }
template <class P> int sqLength(const P& p) { return (p[0]*p[0] + p[1]*p[1] + p[2]*p[2]); }
template <class P> int sqDistance(const P& p, const P& q) { return sqLength(p - q); }

GeometricTransformation::GeometricTransformation(const int p0, const int p1, const int p2)
{
  invperm[p0 < 0 ? -p0-1 : p0-1] = 0; invsign[p0 < 0 ? -p0-1 : p0-1] = (p0 < 0 ? -1 : 1);
  invperm[p1 < 0 ? -p1-1 : p1-1] = 1; invsign[p1 < 0 ? -p1-1 : p1-1] = (p1 < 0 ? -1 : 1);
  invperm[p2 < 0 ? -p2-1 : p2-1] = 2; invsign[p2 < 0 ? -p2-1 : p2-1] = (p2 < 0 ? -1 : 1);
}

GeometricTransformation::GeometricTransformation(const int id)
{
  switch(id >> 3)
  {
    case 0: invperm[0] = 0; invperm[1] = 1; invperm[2] = 2; break;
    case 1: invperm[0] = 0; invperm[1] = 2; invperm[2] = 1; break;
    case 2: invperm[0] = 1; invperm[1] = 0; invperm[2] = 2; break;
    case 3: invperm[0] = 1; invperm[1] = 2; invperm[2] = 0; break;
    case 4: invperm[0] = 2; invperm[1] = 0; invperm[2] = 1; break;
    case 5: invperm[0] = 2; invperm[1] = 1; invperm[2] = 0; break;
  }
  invsign[0] = (id & 0x4 ? -1 : 1);
  invsign[1] = (id & 0x2 ? -1 : 1);
  invsign[2] = (id & 0x1 ? -1 : 1);
}

GeometricTransformation::GeometricTransformation(const GeometricTransformation& t)
{
  for (int i = 0; i < d; ++i) { invperm[i] = t.invperm[i]; invsign[i] = t.invsign[i]; }
}

GeometricTransformation::GeometricTransformation(istream& s)
{
  int p0, p1, p2;
  bool parsingError = false;
  char c1, c2; 
  s >> std::ws >> p0 >> std::ws >> p1 >> std::ws >> p2;
  if (!s.good()) parsingError = true;
  // it's a signed permutation if and only if the squares sum up to 14
  if (p0 * p0 + p1 * p1 + p2 * p2 != 14) parsingError = true;
  if (parsingError) 
  {
    cerr << "Error parsing transformation" << endl;
    exit(1);
  }
  invperm[p0 < 0 ? -p0-1 : p0-1] = 0; invsign[p0 < 0 ? -p0-1 : p0-1] = (p0 < 0 ? -1 : 1);
  invperm[p1 < 0 ? -p1-1 : p1-1] = 1; invsign[p1 < 0 ? -p1-1 : p1-1] = (p1 < 0 ? -1 : 1);
  invperm[p2 < 0 ? -p2-1 : p2-1] = 2; invsign[p2 < 0 ? -p2-1 : p2-1] = (p2 < 0 ? -1 : 1);
}

GeometricTransformation::operator int() const
{
  int index = invperm[0] << 1;
  if (invperm[1] > invperm[2]) index |= 1;
  for (int i = 0; i < d; ++i) { index <<= 1; if (invsign[i] < 0) index |= 1; }
  return index;
}

bool GeometricTransformation::isReflection() const
{
  int moved = 0; for (int i = 0; i < 3; ++i) if (invperm[i] != i) ++moved;
  bool reflection = (moved == 2);
  for (int i = 0; i < d; ++i) if (invsign[i] == -1) reflection = !reflection; 
  return reflection;
}

GeometricTransformation GeometricTransformation::inverse() const
{
  GeometricTransformation t;
  for (int i = 0; i < d; ++i) { t.invperm[invperm[i]] = i; t.invsign[invperm[i]] = invsign[i]; }
  return t;
}

OPt GeometricTransformation::operator()(const OPt& p) const
{
  return OPt(p[invperm[0]]*invsign[0], p[invperm[1]]*invsign[1], p[invperm[2]]*invsign[2]);
}
  
GPt GeometricTransformation::operator()(const GPt& p) const
{
  return GPt(p[invperm[0]]*invsign[0], p[invperm[1]]*invsign[1], p[invperm[2]]*invsign[2]);
}

GeometricTransformation GeometricTransformation::operator*(const GeometricTransformation& t) const
{
  GeometricTransformation c(*this);
  for (int i = 0; i < d; ++i) { c.invperm[i] = t.invperm[invperm[i]]; c.invsign[i] *= t.invsign[invperm[i]]; }
  return c;
}

bool GeometricTransformation::operator==(const GeometricTransformation& t) const
{
  return ((int) (*this) == (int) t);
}

ostream& operator<<(ostream& o, const GeometricTransformation& t)
{
  int p[3]; for (int i = 0; i < d; ++i) p[t.invperm[i]] = (i+1) * t.invsign[i];
  return (o << p[0] << ' ' << p[1] << ' ' << p[2]);
}

void Transformation::parse(istream& s)
{
  bool parsingError = false;
  char c; s >> std::ws >> c;
  if (!s.good()) parsingError = true;
  if (!parsingError) switch(c)
  {
    case '[': direction = 0; break;
    case '{': direction = 7; break;
    default: parsingError = true; break;
  }
  if (!parsingError) geometry = GeometricTransformation(s);
  s >> std::ws >> c; if (!s.good()) parsingError = true; 
  if (!parsingError) switch(c)
  {
    case '}': parsingError = (direction != 0); break;
    case ']': break;
    default: parsingError = true; break;
  }
  if (!parsingError) return;
  cerr << "Error parsing transformation" << endl;
  exit(1);
}

Transformation::Transformation(const GeometricTransformation& theGeometry, const int theDirection):
  geometry(theGeometry), direction(theDirection)
{};

Transformation::Transformation(const int id):
  geometry(id % 48), direction(id < 48 ? 0 : 7)
{};

Transformation::Transformation(istream& s)
{
  parse(s);
}

Transformation::Transformation(const string& s)
{
  stringstream i(s);
  parse(i);
}

/*
int Transformation::id() const
{
  return ((direction == 0 ? 0 : 48) + (int) geometry);
}
*/

Transformation Transformation::inverse() const
{
  return Transformation(geometry.inverse(), direction);
}

Transformation Transformation::operator*(const Transformation& t) const
{
  return Transformation(geometry * t.geometry, direction ^ t.direction);
}

ostream& operator<<(ostream& o, const Transformation& t)
{
  return o 
    << (t.direction == 0 ? '[' : '{') << ' ' 
    << t.geometry << ' ' 
    << (t.direction == 0 ? '}' : ']');
}

Symmetry::Symmetry(const char theSymbol, const int p0, const int p1, const int p2):
  Transformation(GeometricTransformation(p0, p1, p2), 7), symbol(theSymbol)
{};

Symmetries::Symmetries()
{
  push_back(Symmetry('a',  1,  2, -3));
  push_back(Symmetry('b',  1, -2,  3));
  push_back(Symmetry('c', -1,  2,  3));
  push_back(Symmetry('d',  1, -2, -3));
  push_back(Symmetry('e', -1,  2, -3));
  push_back(Symmetry('f', -1, -2,  3));
  push_back(Symmetry('g', -1,  3,  2));
  push_back(Symmetry('h', -1, -3, -2));
  push_back(Symmetry('i',  3, -2,  1));
  push_back(Symmetry('j', -3, -2, -1));
  push_back(Symmetry('k',  2,  1, -3));
  push_back(Symmetry('l', -2, -1, -3));
  push_back(Symmetry('o', -1, -2, -3));
  push_back(Symmetry('q',  1,  3, -2));
  push_back(Symmetry('r',  3,  2, -1));
  push_back(Symmetry('s',  2, -1,  3));
  push_back(Symmetry('t', -1,  3, -2));
  push_back(Symmetry('u',  2, -1, -3));
  push_back(Symmetry('v',  3, -2, -1));
  push_back(Symmetry('w', -3, -1, -2));
  push_back(Symmetry('x',  3, -1,  2));
  push_back(Symmetry('y', -3,  1,  2));
  push_back(Symmetry('z',  3,  1, -2));
}

