/*******************************************************************
HILBEX
version 0.1, 1 Oct 2016

This file is part of the software package Hilbex by Herman
Haverkort. Use and distribution of this software is subject to
restrictions. Consult readme.txt for details, or download it from
http://spacefillingcurves.net/.
*******************************************************************/

Options::Options(const int nrWords, char* argv[]):
  filter(), maxOutput(1), countCurves(true), 
  outputNames(None), outputProperties(None), outputDefinition(false), 
  traversalOrder(0), transformation(), randomTransformation(false), 
  basePatternVerificationNumbers(false),
  povRayStyle("blackwire.inc"), povRayGap(0.13), povRayContraction(0), povRayOutput(false),
  testLevel(Concise), outputNotes(false), batched(false),
  progressIndicator(true), outputWarnings(true)
{
  vector<string> word; for (int i = 0; i < nrWords; ++i) word.push_back(string(argv[i]));
  
  bool qualified = false; OutputLevel qualifier;
  for (int i = 0; i < nrWords; ++i)
  {
    if (word[i] == "no" || word[i] == "concise" || word[i] == "full") 
    {
      if (qualified) giveUp(string("unexpected \"") + word[i] + "\" after \"" + word[i-1] + "\".");
      if (word[i] == "no") qualifier = None;
      else if (word[i] == "concise") qualifier = Concise;
      else qualifier = Full;
      qualified = true;
    }
    else if (word[i] == "filter")
    {
      if (++i == nrWords) giveUp(string("missing filter definition"));
      filter = Filter(word[i], filter);
      if (!filter.consistent()) giveUp(string("invalid filter."));
      qualified = false;
    }
    else if (word[i] == "output")
    {
      if (++i == nrWords) giveUp(string("missing number of curves to output"));
      if (word[i] == "all") maxOutput = 20000000; // more than enough
      else maxOutput = atoi(word[i].c_str());
      qualified = false;
    }
    else if (word[i] == "count")
    {
      if (qualified && qualifier == Concise) giveUp(string("unexpected \"") + word[i] + "\" after \"" + word[i-1] + "\".");
      countCurves = (qualifier == Full || !qualified); 
      qualified = false;
    }
    else if (word[i] == "name" || word[i] == "names")
    {
      outputNames = (qualified ? qualifier : Concise);
      qualified = false;
    }
    else if (word[i] == "properties") 
    {
      outputProperties = (qualified ? qualifier : Full);
      qualified = false;
    }
    else if (word[i] == "definition")
    {
      if (qualified && qualifier == Concise) giveUp(string("unexpected \"") + word[i] + "\" after \"" + word[i-1] + "\".");
      outputDefinition = (qualifier == Full || !qualified); 
      qualified = false;
    }
    else if (word[i] == "order")
    {
      if (++i == nrWords) giveUp(string("missing order of approximating curve point sequence to output"));
      traversalOrder = atoi(word[i].c_str());
      qualified = false;
    }
    else if (word[i] == "transformation")
    {
      if (++i == nrWords) giveUp(string("missing transformation to apply"));
      randomTransformation = (word[i] == "random");
      if (!randomTransformation) transformation = Transformation(word[i]);
      qualified = false;
    }
    else if (word[i] == "patterns")
    {
      basePatternVerificationNumbers = true;
      qualified = false;
    }
    else if (word[i] == "drawing")
    {
      if (qualified && qualifier == Concise) giveUp(string("unexpected \"") + word[i] + "\" after \"" + word[i-1] + "\".");
      povRayOutput = true;
      qualified = false;
    }
    else if (word[i] == "style")
    {
      if (++i == nrWords) giveUp(string("missing POVRay style file name"));
      povRayStyle = word[i];
      if (povRayStyle.find('.') == string::npos) povRayStyle += ".inc";
      qualified = false;
    }
    else if (word[i] == "gap")
    {
      if (++i == nrWords) giveUp(string("missing bisector gap size (relative to cube width) for POVRay model"));
      povRayGap = atof(word[i].c_str());
      qualified = false;
    }
    else if (word[i] == "contract")
    {
      if (++i == nrWords) giveUp(string("missing contraction factor for POVRay model"));
      povRayContraction = atof(word[i].c_str());
      qualified = false;
    }
    else if (word[i] == "test" || word[i] == "tests")
    {
      testLevel = (qualified ? qualifier : Concise);
      qualified = false;
    }      
    else if (word[i] == "notes")
    {
      if (qualified && qualifier == Concise) giveUp(string("unexpected \"") + word[i] + "\" after \"" + word[i-1] + "\".");
      outputNotes = (qualifier == Full || !qualified); 
      qualified = false;
    }
    else if (word[i] == "batched")
    {
      if (qualified) giveUp(string("unexpected \"") + word[i] + "\" after \"" + word[i-1] + "\".");
      batched = true;
      qualified = false;
    }
    else if (word[i] == "progress")
    {
      if (qualified && qualifier == Concise) giveUp(string("unexpected \"") + word[i] + "\" after \"" + word[i-1] + "\".");
      progressIndicator = (qualifier == Full || !qualified); 
      qualified = false;
    }
    else if (word[i] == "warning" || word[i] == "warnings")
    {
      if (qualified && qualifier == Concise) giveUp(string("unexpected \"") + word[i] + "\" after \"" + word[i-1] + "\".");
      outputWarnings = (qualifier == Full || !qualified); 
      qualified = false;
    }
    else
    {  
      giveUp(string("unrecognized option \"") + word[i] + "\".");
    }
  }
  if (maxOutput == 0 && countCurves == false) giveUp(string("with these options, no output is required."));
  if (povRayOutput)
  {
    if (maxOutput != 1 || batched) 
      giveUp(string("POVRay output can only be generated in non-batched mode for exactly one curve."));
    if ((countCurves || outputNames != None || outputProperties != None || outputDefinition || outputNotes) && outputWarnings) 
      cerr << "Warning: POVRay output is requested; all other output to standard output will be suppressed." << endl;
    if (traversalOrder < 1)
      giveUp(string("For POVRay output, please specify the order of the approximating curve (at least 1) to be output."));
    if (traversalOrder > 5)
    {
      cerr << "The specified options ask for a " << (1 << (3 * traversalOrder)) << "-point approximating curve\nto be output as POVRay model. "; 
      giveUp(string("That cannot be right."));
    }
    countCurves = false;
    outputNames = None;
    outputProperties = None;
    outputDefinition = false;
    outputNotes = false;
  }  
  if (basePatternVerificationNumbers)
  {
    if (batched) giveUp(string("Base pattern identifiers can only be generated for all base patterns. Filters specified on the command line or in input files are not supported."));
    cerr << "Generating identifiers for all possible base patterns according to canonical numbering for verification. All other output will be suppressed. Any filters specified on the command line will be ignored." << endl;
    filter = Filter();
    countCurves = false;
  }
  
  if (traversalOrder > 4 && outputWarnings)
  {
    cerr << endl
         << "The specified options ask for a " << (1 << (3 * traversalOrder)) << "-point approximating curve" << endl
         << "for each curve. That is a lot. Please rerun with option \"no warnings\"" << endl
         << "if you really want this." << endl;
    giveUp(string("just checking to be sure."));
  }
}

void Options::giveUp(const string& message)
{
  cerr << "Error parsing options: " << message << endl;
  exit(1);
}
