/*******************************************************************
HILBEX
version 0.1, 1 Oct 2016

This file is part of the software package Hilbex by Herman
Haverkort. Use and distribution of this software is subject to
restrictions. Consult readme.txt for details, or download it from
http://spacefillingcurves.net/.
*******************************************************************/

#ifndef HILBERT3DSUITE_OPTIONS
#define HILBERT3DSUITE_OPTIONS

#include "filter.h"

enum OutputLevel { None, Concise, Full };

struct Options
{
  Filter filter; 
  int maxOutput; // the maximum number of curves output; default is 1
  bool countCurves; // whether to count all curves that qualify, or stop after finding maxOutput; default is true
  OutputLevel outputNames; // none, short names, or long names; default is none
  OutputLevel outputProperties; // none, table, or words; default is none
  bool outputDefinition; // whether to output the definition of the curve; default is false;
  int traversalOrder; // what order of approximating curve to output; default is 0 (no approximating curve);
  Transformation transformation; // what transformation to use for definition and approximating curve; default is identity;
  bool randomTransformation; // whether to apply a random transformation; default is false;
  bool basePatternVerificationNumbers; // whether to apply base pattern verification numbers (see Appendix C) instead of any other output
  bool povRayOutput; // whether to output a POVRay model (instead of other output)
  string povRayStyle; // povRay include file to use for povRay-output; default is "blackwire.inc";
                      // alternatives are "blueorangewire.inc" (colour gradient blue-purple-red-orange)
                      // and "stairs.inc" (building style, only works for some curves, see documentation)
  double povRayGap; // what fraction of width to use for extra spacing around bisector; default is 0.13;
  double povRayContraction; // how far to contract the curve to the front (for "animating" harmony); default is 0;
  OutputLevel testLevel; // what tests to run; 
    // if none, only the simples properties are tested for, there will be no report on 
    // diagonal-harmony and pattern-isotropy, unless the filter requires this;
    // face-continuity, hyperorthogonality, and facet-harmony are hardcoded and not verified;
    // if concise (default), diagonal-harmony and pattern-isotropy are determined; facet-harmony is verified;
    // if full, face-continuity and hyperorthogonality are verified.
  bool outputNotes; // whether to output additional information from curveinfo.txt; default is false
  bool batched; // whether to process filters (one on each line) from standard input; default is false
  bool progressIndicator; // whether to output a progress indicator on the standard error output; default is true  
                          // for non-batched mode: base patterns; for batched mode: number of lines processed
  bool outputWarnings; // default is true
  Options(const int nrWords = 0, char* argv[] = 0);
  
  private:
  void giveUp(const string& message);
};

#endif

