/*******************************************************************
HILBEX
version 0.1, 1 Oct 2016

This file is part of the software package Hilbex by Herman
Haverkort. Use and distribution of this software is subject to
restrictions. Consult readme.txt for details, or download it from
http://spacefillingcurves.net/.
*******************************************************************/

const string Properties::acronym[21] =
{
  "Co", "Ho", "Sy", "Ms", "Fh", "Dh", "Wf",
  "Vg", "Ve", "Vf", "Eg", "Fg", "Ec", "Fc", "Cc",
  "Cr", "Op", "Pi", "Sh", "St", "Xx"
};

const string Properties::name[21] = 
{
  "face-continuous", "hyperorthogonal", "symmetric", "metasymmetric", 
  "fivefold-facet-harmonious", "interior-diagonal-harmonious", "well-folded",
  "vertex-gated", "vertex-edge-gated", "vertex-facet-gated", "edge-gated", "facet-gated",
  "edge-crossing", "facet-crossing", "cube-crossing",
  "centred", "order-preserving", "pattern-isotropic", "coordinate-shifting", "standing",
  "invalid"
};

Property Properties::parse(string word)
{
  for (int p = 0; p <= (int) Invalid; ++p)
    if (name[p] == word) return (Property) p;
  return Invalid;
}

Properties::Properties() {};

void Properties::confirm(Property p)
{
  confirmed.set((int) p);
  rejected.reset((int) p);
}

void Properties::confirm(string p)
{
  confirm(parse(p));
}

void Properties::reject(Property p)
{
  confirmed.reset((int) p);
  rejected.set((int) p);
}

void Properties::reject(string p)
{
  reject(parse(p));
}

bool Properties::is(Property p) const
{
  return confirmed[(int) p];
}

bool Properties::isNot(Property p) const
{
  return rejected[(int) p];
}

string Properties::table() const
{
  stringstream s;
  for (int i = 0; i < (int) Invalid; ++i) 
  {
    if (is((Property) i)) s << acronym[i];
    else if (isNot((Property) i)) s << "--";
    else s << "??";
  }
  return s.str();
}

string Properties::report() const
{
  stringstream s;
  bool comma = false;
  s << "This curve is:"; 
  for (int i = 0; i < (int) Invalid; ++i)
    if (is((Property) i)) { if (comma) s << ','; s << ' ' << name[i]; comma = true; }
  s << "." << endl;
  comma = false;
  s << "This curve is NOT:"; 
  for (int i = (int) FaceContinuous; i < (int) VertexGated; ++i)
    if (!is((Property) i)) { if (comma) s << ','; s << ' ' << name[i]; comma = true; }
  for (int i = (int) Centred; i < (int) Invalid; ++i)
    if (!is((Property) i)) { if (comma) s << ','; s << ' ' << name[i]; comma = true; }
  s << "." << endl;
  return s.str();
}


