/*******************************************************************
HILBEX
version 0.1, 1 Oct 2016

This file is part of the software package Hilbex by Herman
Haverkort. Use and distribution of this software is subject to
restrictions. Consult readme.txt for details, or download it from
http://spacefillingcurves.net/.
*******************************************************************/

#ifndef HILBERT3DSUITE_SEARCH
#define HILBERT3DSUITE_SEARCH

#include "basepatterns.h" 
#include "geometry.h"
#include "curve.h" 
#include "filter.h" 

class Hilbert3DCurve;

class Hilbert3DCurveSearch
{
  private:
    vector<Partition>::const_iterator partition;
    Symmetries::const_iterator transformation;
    vector<Permutation>::const_iterator permutation[2];
    vector<GateType>::const_iterator gateType[2];
    int connections[2];
    int reversals[2];
    int reflections[2];
    
    bool vertexGated() const;
    bool vertexEdgeGated() const;
    bool vertexFacetGated() const;
    bool edgeGated() const;
    bool facetGated() const;
    bool symmetricBasePattern() const;

    bool findValidBasePattern();
    bool findValidGates();
    bool verifyGatesEdgeCrossingVertexGatedCurve(int i);
    bool verifyGatesFacetCrossingVertexGatedCurve(int i);
    bool verifyGatesVertexGatedCurve(int i);
    bool findValidVertexGatedCurve();
    bool findValidVertexFacetGatedCurve();
    bool verifyGatesReflectionDeterminedCurve(int i);
    bool findValidReflectionDeterminedCurve();

    string basePatternName;
    OPt localPattern[2][5]; // half-patterns from local perspective, per side
    GPt localGate[2][5]; // gates from local perspective, per side, with coordinates in multiples of 1/6
    Transformation localTransformationPattern[2][4];
    GPt gate[9];

    void calculateLocalTransformations();

    void calculateName(Hilbert3DCurve& h) const;
    void calculateDescription(Hilbert3DCurve& h);

    bool progressIndicator;
    bool patternIds;
    
  public:
    Filter filter;
    Hilbert3DCurveSearch(Filter f = Filter(), bool showProgress = false, bool onlyPatternIds = false);
    bool findValidCurve(Hilbert3DCurve& h);
    bool skip();
};

#endif

